@@ -29,17 +29,22 @@ to share a single outer scope. E.g.
 
     def outer(a):
         def inner(b):
-            def closure():
+            def inner_inner():
                 print a, b
-            return closure
+            return inner_inner
 
         return inner(1), inner(2)
 
-We have three closure scopes here:
+We have three live closure scopes here:
 
-    scope_outer = { 'a': a }
-    scope_inner_1 = { 'scope_outer': scope_outer, 'b': 1 }
-    scope_inner_2 = { 'scope_outer': scope_outer, 'b': 2 }
+    scope_outer = { 'a': a }  # call to 'outer'
+    scope_inner_1 = { 'scope_outer': scope_outer, 'b': 1 } # call to 'inner' with b=1
+    scope_inner_2 = { 'scope_outer': scope_outer, 'b': 2 } # call to 'inner' with b=2
+
+Function 'inner_inner' defines no new scope, since it contains no cellvars.
+But it does contain a freevar from scope_outer and scope_inner, so it gets
+scope_inner passed as first argument. scope_inner has a reference to scope
+outer, so all variables can be resolved.
 
 These scopes are instances of a numba extension class.
 """
@@ -47,12 +52,14 @@ These scopes are instances of a numba extension class.
 import ast
 import types
 import ctypes
+import inspect
 
+import llvm.core as lc
 import numba.decorators
 from numba import *
-from numba import error, visitors, nodes
+from numba import error, visitors, nodes, module_type_inference
 from numba.minivect import  minitypes
-from numba import  _numba_types as numba_types
+from numba import typesystem
 from numba.symtab import Variable
 
 import logging
@@ -68,12 +75,6 @@ class ClosureMixin(object):
     do not recurse into the inner functions themselves!
     """
 
-    def _visit_func_children(self, node):
-        self.function_level += 1
-        self.generic_visit(node)
-        self.function_level -= 1
-        return node
-
     def _err_decorator(self, decorator):
         raise error.NumbaError(
                 decorator, "Only @jit and @autojit and signature decorators "
@@ -125,9 +126,7 @@ class ClosureMixin(object):
         return restype
 
     def _handle_jit_decorator(self, func_def, decorator):
-        from numba import ast_type_inference
-
-        jit_args = ast_type_inference._parse_args(
+        jit_args = module_type_inference.parse_args(
                 decorator, ['restype', 'argtypes', 'backend',
                             'target', 'nopython'])
 
@@ -194,23 +193,25 @@ class ClosureMixin(object):
         del node.decorator_list[:]
 
         if len(signature.args) != len(node.args.args):
-            self.error(decorator,
-                       "Expected %d arguments type(s), got %d" % (
+            raise error.NumbaError(
+                decorator,
+                "Expected %d arguments type(s), got %d" % (
                                 len(signature.args), len(node.args.args)))
 
         return signature
 
     def visit_FunctionDef(self, node):
         if self.function_level == 0:
-            return self._visit_func_children(node)
+            return self.visit_func_children(node)
 
         signature = self._process_decorators(node)
-        type = numba_types.ClosureType(signature)
+        type = typesystem.ClosureType(signature)
         self.symtab[node.name] = Variable(type, is_local=True)
 
         closure = nodes.ClosureNode(node, type, self.func)
         type.closure = closure
         self.ast.closures.append(closure)
+        self.closures[node.name] = closure
 
         return closure
 
@@ -241,11 +242,15 @@ def lookup_scope_attribute(cur_scope, var_name, ctx=None):
     elif outer_scope_type.is_closure_scope:
         scope = nodes.ExtTypeAttribute(value=cur_scope, attr=outer_scope_name,
                                        ctx=ctx, ext_type=scope_type)
-        return lookup_scope_attribute(scope, var_name, ctx)
-    else:
-        # This indicates a bug
-        raise error.InternalError(
-                cur_scope, "Unable to look up attribute %s" % var_name)
+        try:
+            return lookup_scope_attribute(scope, var_name, ctx)
+        except error.InternalError, e:
+            # Re-raise with full scope type
+            pass
+
+    # This indicates a bug
+    raise error.InternalError(
+            scope_type, "Unable to look up attribute", var_name)
 
 CLOSURE_SCOPE_ARG_NAME = '__numba_closure_scope'
 
@@ -271,20 +276,19 @@ class ClosureTypeInferer(ClosureBaseVisitor, visitors.NumbaTransformer):
     3) generate nodes to instantiate scope extension type at call time
     """
 
-    func_level = 0
-
-    def _visit_children(self, node):
-        self.func_level += 1
-        self.generic_visit(node)
-        self.func_level -= 1
-        return node
+    def __init__(self, *args, **kwargs):
+        super(ClosureTypeInferer, self).__init__(*args, **kwargs)
+        self.warn = kwargs["warn"]
 
     def visit_FunctionDef(self, node):
         if node.closure_scope is None:
             # Process inner functions and determine cellvars and freevars
             # codes = [c for c in self.constants
             #                if isinstance(c, types.CodeType)]
-            process_closures(self.context, node, self.symtab)
+            process_closures(self.context, node, self.symtab,
+                             func_globals=self.func_globals,
+                             closures=self.closures,
+                             warn=self.warn)
 
         # cellvars are the variables we own
         cellvars = dict((name, var) for name, var in self.symtab.iteritems()
@@ -304,7 +308,7 @@ class ClosureTypeInferer(ClosureBaseVisitor, visitors.NumbaTransformer):
             if outer_scope:
                 self.update_closures(node, outer_scope_type, None)
 
-            return self._visit_children(node)
+            return self.visit_func_children(node)
 
         # Create closure scope extension type
         cellvar_fields = [(name, var.type)
@@ -317,9 +321,9 @@ class ClosureTypeInferer(ClosureBaseVisitor, visitors.NumbaTransformer):
         class py_class(object):
             pass
 
-        func_name = self.func.__name__
+        func_name = self.func_name
         py_class.__name__ = '%s_scope' % func_name
-        scope_type = numba_types.ClosureScopeType(py_class, outer_scope_type)
+        scope_type = typesystem.ClosureScopeType(py_class, outer_scope_type)
         scope_type.unmangled_symtab = dict(fields)
 
         mangled_fields = [(mangle(name, scope_type), type)
@@ -340,10 +344,12 @@ class ClosureTypeInferer(ClosureBaseVisitor, visitors.NumbaTransformer):
         node.body.insert(0, cellvar_scope)
 
         self.update_closures(node, scope_type, ext_type)
-        return self._visit_children(node)
+        return self.visit_func_children(node)
 
     def update_closures(self, func_def, scope_type, ext_type):
-        # Patch closures to get the closure scope as the first argument
+        """
+        Patch closures to get the closure scope as the first argument.
+        """
         for closure in func_def.closures:
             # closure.scope_type = scope_type
             closure.func_def.scope_type = scope_type
@@ -361,16 +367,12 @@ class ClosureTypeInferer(ClosureBaseVisitor, visitors.NumbaTransformer):
             # patch closure signature
             closure.type.signature.args = (scope_type,) + closure.type.signature.args
 
-#    def visit_ClosureNode(self, node):
-#        node.func_def = self.visit(node.func_def)
-#        return node
-
 
 def get_locals(symtab):
     return dict((name, var) for name, var in symtab.iteritems()
                     if var.is_local)
 
-def process_closures(context, outer_func_def, outer_symtab):
+def process_closures(context, outer_func_def, outer_symtab, **kwds):
     """
     Process closures recursively and for each variable in each function
     determine whether it is a freevar, a cellvar, a local or otherwise.
@@ -388,19 +390,22 @@ def process_closures(context, outer_func_def, outer_symtab):
     for closure in outer_func_def.closures:
         logger.debug("process closures: %s %s", outer_func_def.name,
                      closure.func_def.name)
-        closure.make_pyfunc()
-        symtab = {}
+        # closure.make_pyfunc()
+        closure_py_func = None # closure.py_func
+        #symtab = {}
         p, result = numba.pipeline.infer_types_from_ast_and_sig(
-                    context, closure.py_func, closure.func_def,
+                    context, closure_py_func, closure.func_def,
                     closure.type.signature,
                     closure_scope=closure_scope,
-                    symtab=symtab)
+                    locals=closure.locals,
+                    is_closure=True,
+                    **kwds)
 
         _, _, ast = result
-        closure.symtab = symtab
+        closure.symtab = p.symtab
         closure.type_inferred_ast = ast
 
-        process_closures(context, closure.func_def, symtab)
+        process_closures(context, closure.func_def, p.symtab, **kwds)
 
 
 class ClosureCompilingMixin(ClosureBaseVisitor):
@@ -456,6 +461,7 @@ class ClosureCompilingMixin(ClosureBaseVisitor):
             if arg.id in node.scope_type.unmangled_symtab:
                 dst = lookup_scope_attribute(scope, arg.id, ast.Store())
                 src = self._load_name(arg.id)
+                src.variable.assign_in_closure_scope = True
                 assmt = ast.Assign(targets=[dst], value=src)
                 stats.append(assmt)
 
@@ -470,12 +476,19 @@ class ClosureCompilingMixin(ClosureBaseVisitor):
         # Compile inner function, skip type inference
         order = numba.pipeline.Pipeline.order
         order = order[order.index('type_infer') + 1:]
+
+        ns = '.'.join([self.module_name, self.func_name])
+        closure_name = node.name
+        fullname = "%s.__closure__.%s" % (ns, closure_name)
+
         p, result = numba.pipeline.run_pipeline(
-                    self.context, node.py_func, node.type_inferred_ast,
+                    self.context, None, node.type_inferred_ast,
                     node.type.signature, symtab=node.symtab,
                     order=order, # skip type inference
+                    qualified_name=fullname,
+                    locals=node.locals,
                     )
-
+        p.translator.link()
         node.lfunc = p.translator.lfunc
         node.lfunc_pointer = p.translator.lfunc_pointer
 
@@ -485,7 +498,18 @@ class ClosureCompilingMixin(ClosureBaseVisitor):
             func_name = node.func_def.name
             self.symtab[func_name] = Variable(name=func_name, type=node.type,
                                               is_local=True)
-            return ast.Pass()
+            # return nodes.LLVMValueRefNode(node.type, node.lfunc)
+            # TODO: Remove assignment altogether!
+            # return nodes.NoneNode()
+            return nodes.ObjectInjectNode(None, type=object_)
+
+    def assign_closure(self, func_call, node):
+        "Assign closure to its name. NOT USED, already happened in CFG"
+        func_name = node.func_def.name
+        dst = self._load_name(func_name, self.symtab[func_name].is_cellvar)
+        dst.ctx = ast.Store()
+        result = ast.Assign(targets=[dst], value=func_call)
+        return result
 
     def create_numba_function(self, node, p):
         closure_scope = self.ast.cur_scope
@@ -501,9 +525,9 @@ class ClosureCompilingMixin(ClosureBaseVisitor):
                     p.translator.build_wrapper_function(get_lfunc=True))
 
         # Keep methoddef alive
-        assert methoddef in node.py_func.live_objects
-        modname = self.func.__module__
-        node.py_func.live_objects.append(modname)
+        # assert methoddef in node.py_func.live_objects
+        modname = self.module_name
+        self.keep_alive(modname)
 
         # Create function with closure scope at runtime
         create_numbafunc_signature = node.type(
@@ -529,7 +553,7 @@ class ClosureCompilingMixin(ClosureBaseVisitor):
             closure_scope,
             nodes.const(node.lfunc_pointer, void.pointer()),
             nodes.const(node.type.signature, object_),
-            nodes.const(node.py_func, object_),
+            nodes.NULL, # nodes.const(node.py_func, object_),
         ]
 
         func_call = nodes.NativeFunctionCallNode(
@@ -537,11 +561,8 @@ class ClosureCompilingMixin(ClosureBaseVisitor):
                             function_node=create_numbafunc,
                             args=args)
 
-        # Assign closure to its name
-        func_name = node.func_def.name
-        dst = self._load_name(func_name, self.symtab[func_name].is_cellvar)
-        dst.ctx = ast.Store()
-        result = ast.Assign(targets=[dst], value=func_call)
+        # result = self.assign_closure(func_call, node)
+        result = func_call
 
         #stats = [nodes.inject_print(nodes.const("calling...", c_string_type)),
         #         result]
@@ -570,4 +591,4 @@ class ClosureCompilingMixin(ClosureBaseVisitor):
         return node
 
     def visit_ClosureScopeLoadNode(self, node):
-        return self.ast.cur_scope or nodes.NULL_obj
\ No newline at end of file
+        return self.ast.cur_scope or nodes.NULL_obj
