@@ -99,7 +99,7 @@ class LLVMCodeGen(codegen.CodeGen):
 
         llvm_fpm = llvm.passes.FunctionPassManager.new(self.llvm_module)
         # target_data = llvm.ee.TargetData(self.context.llvm_ee)
-        llvm_fpm.add(self.context.llvm_ee.target_data)
+        llvm_fpm.add(self.context.llvm_ee.target_data.clone())
         pmb = llvm.passes.PassManagerBuilder.new()
         pmb.opt_level = 3
         pmb.vectorize = True
@@ -422,7 +422,8 @@ class LLVMCodeGen(codegen.CodeGen):
         return self.visit_SingleIndexNode(node)
 
     def visit_SizeofNode(self, node):
-        return self.visit(self.astbuilder.constant(node.sizeof_type.itemsize))
+        return self.visit(self.astbuilder.constant(node.sizeof_type.itemsize,
+                                                   node.type))
 
     def visit_Variable(self, node):
         value = self.symtab[node.name]
@@ -512,14 +513,9 @@ class LLVMCodeGen(codegen.CodeGen):
             return llvm_call
 
     def visit_FuncNameNode(self, node):
-        try:
-            func = self.context.llvm_module.get_function_named(node.name)
-        except llvm.LLVMException:
-            # Define as external function
-            assert node.is_external
-            func_type = node.type.to_llvm(self.context)
-            func = self.context.llvm_module.add_function(func_type, node.name)
-
+        func_type = node.type.to_llvm(self.context)
+        func = self.context.llvm_module.get_or_insert_function(func_type,
+                                                               node.name)
         return func
 
     def visit_FuncRefNode(self, node):
