@@ -183,9 +183,14 @@ class Context(object):
             codegen = self.codegen_cls(self, codewriter)
             codegen.visit(specialized_ast)
 
-            yield (pipeline[0], specialized_ast, codewriter,
+            specializer = pipeline[0]
+            yield (specializer, specialized_ast, codewriter,
                    self.codeformatter_cls().format(codewriter))
 
+    def run_simple(self, ast, specializer):
+        (_, _, _, code_result), = self.run(ast, [specializer])
+        return code_result
+
     def debug_c(self, ast, specializer, astbuilder_cls=None):
         "Generate C code (for debugging)"
         context = CContext()
@@ -270,6 +275,8 @@ class Context(object):
 
     def promote_types(self, type1, type2):
         "Promote types in an arithmetic operation"
+        if type1 == type2:
+            return type1
         return self.typemapper.promote_types(type1, type2)
 
     def getchildren(self, node):
@@ -302,6 +309,9 @@ class Context(object):
         graphviz_graph = visitor.visit(node)
         return graphviz_graph.to_string()
 
+    def is_object(self, type):
+        return isinstance(type, minitypes.ObjectType)
+
 class CContext(Context):
     "Set defaults for C code generation."
 
@@ -1266,9 +1276,12 @@ class NodeWrapper(ExprNode):
     def __deepcopy__(self, memo):
         kwds = dict(vars(self))
         kwds.pop('opaque_node')
+        kwds.pop('specialize_node_callback')
         kwds = copy.deepcopy(kwds, memo)
         opaque_node = self.specialize_node_callback(self, memo)
-        return type(self)(opaque_node=opaque_node, **kwds)
+        return type(self)(opaque_node=opaque_node,
+                          specialize_node_callback=self.specialize_node_callback,
+                          **kwds)
 
 class BinaryOperationNode(ExprNode):
     "Base class for binary operations"
