@@ -73,11 +73,11 @@ class TypeMapper(object):
     PyObject *
 
     >>> tm.promote_types(int64, float32)
-    float
+    float32
     >>> tm.promote_types(int64, complex64)
     complex64
     >>> tm.promote_types(float32, float64)
-    double
+    float64
     >>> tm.promote_types(float32, complex64)
     complex64
     >>> tm.promote_types(complex64, complex128)
@@ -86,27 +86,39 @@ class TypeMapper(object):
     PyObject *
 
     >>> tm.promote_types(float32.pointer(), Py_ssize_t)
-    float *
+    float32 *
     >>> tm.promote_types(float32.pointer(), Py_ssize_t)
-    float *
+    float32 *
     >>> tm.promote_types(float32.pointer(), uint8)
-    float *
+    float32 *
 
     >>> tm.promote_types(float32.pointer(), float64.pointer())
     Traceback (most recent call last):
         ...
-    UnpromotableTypeError: (float *, double *)
+    UnpromotableTypeError: (float32 *, float64 *)
 
     >>> tm.promote_types(float32[:, ::1], float32[:, ::1])
-    float[:, ::1]
+    float32[:, ::1]
     >>> tm.promote_types(float32[:, ::1], float64[:, ::1])
-    double[:, ::1]
+    float64[:, ::1]
     >>> tm.promote_types(float32[:, ::1], float64[::1, :])
-    double[:, :]
+    float64[:, :]
     >>> tm.promote_types(float32[:, :], complex128[:, :])
     complex128[:, :]
     >>> tm.promote_types(int_[:, :], object_[:, ::1])
     PyObject *[:, :]
+
+    >>> tm.promote_types(npy_intp, float_)
+    float
+    >>> tm.promote_types(npy_intp, float32)
+    float32
+
+    >>> float32 == float_
+    True
+    >>> float64 == double
+    True
+    >>> float128 == longdouble
+    True
     """
 
     def __init__(self, context):
@@ -139,6 +151,8 @@ class TypeMapper(object):
 
         if isinstance(value, float):
             return double
+        elif isinstance(value, bool):
+            return bool_
         elif isinstance(value, (int, long)):
             return int_
         elif isinstance(value, complex):
@@ -186,6 +200,7 @@ class TypeMapper(object):
         else:
             raise minierror.UnpromotableTypeError((type1, type2))
 
+
 def map_dtype(dtype):
     """
     Map a NumPy dtype to a minitype.
@@ -198,7 +213,7 @@ def map_dtype(dtype):
     >>> map_dtype(np.dtype(np.object))
     PyObject *
     >>> map_dtype(np.dtype(np.float64))
-    double
+    float64
     >>> map_dtype(np.dtype(np.complex128))
     complex128
     """
@@ -331,6 +346,9 @@ class Type(miniutils.ComparableObjectMixin):
 
     subtypes = []
 
+    mutated = True
+    _ctypes_type = None
+
     def __init__(self, **kwds):
         vars(self).update(kwds)
         self.qualifiers = kwds.get('qualifiers', frozenset())
@@ -367,15 +385,23 @@ class Type(miniutils.ComparableObjectMixin):
     def __eq__(self, other):
         # Don't use isinstance here, compare on exact type to be consistent
         # with __hash__. Override where sensible
+        cmps = self.comparison_type_list
+        if not cmps:
+            return id(self) == id(other)
+
         return (type(self) is type(other) and
-                self.comparison_type_list == other.comparison_type_list)
+                cmps == other.comparison_type_list)
 
     def __ne__(self, other):
         return not self == other
 
     def __hash__(self):
+        cmps = self.comparison_type_list
+        if not cmps:
+            return hash(id(self))
+
         h = hash(type(self))
-        for subtype in self.comparison_type_list:
+        for subtype in cmps:
             h = h ^ hash(subtype)
 
         return h
@@ -421,8 +447,16 @@ class Type(miniutils.ComparableObjectMixin):
         return context.to_llvm(self)
 
     def to_ctypes(self):
-        import ctypes_conversion
-        return ctypes_conversion.convert_to_ctypes(self)
+        """
+        Convert type to ctypes. The result may be cached!
+        """
+        if self._ctypes_type is None:
+            import ctypes_conversion
+            self._ctypes_type = ctypes_conversion.convert_to_ctypes(self)
+            self.mutated = False
+
+        assert not self.mutated
+        return self._ctypes_type
 
     def get_dtype(self):
         return map_minitype_to_dtype(self)
@@ -440,6 +474,14 @@ class Type(miniutils.ComparableObjectMixin):
         """
         return FunctionType(self, args)
 
+class KeyHashingType(Type):
+
+    def __hash__(self):
+        return hash(self.key)
+
+    def __eq__(self, other):
+        return hasattr(other, 'key') and self.key == other.key
+
 class ArrayType(Type):
     """
     An array type. ArrayType may be sliced to obtain a subtype:
@@ -483,16 +525,21 @@ class ArrayType(Type):
 
     def __repr__(self):
         axes = [":"] * self.ndim
-        if self.is_c_contig:
+        if self.is_c_contig and self.ndim > 0:
             axes[-1] = "::1"
-        elif self.is_f_contig:
+        elif self.is_f_contig and self.ndim > 0:
             axes[0] = "::1"
 
         return "%s[%s]" % (self.dtype, ", ".join(axes))
 
+    def copy(self, **kwargs):
+        array_type = copy.copy(self)
+        vars(array_type).update(kwargs)
+        return array_type
+
     @property
     def strided(self):
-        type = copy.copy(self)
+        type = self.copy()
         type.is_c_contig = False
         type.is_f_contig = False
         type.inner_contig = False
@@ -596,10 +643,13 @@ class BoolType(NamedType):
     name = "bool"
 
     def __repr__(self):
-        return "int %s" % " ".join(self.qualifiers)
+        return ("int %s" % " ".join(self.qualifiers)).rstrip()
+
+    def __str__(self):
+        return ("bool %s" % " ".join(self.qualifiers)).rstrip()
 
     def to_llvm(self, context):
-        return int8.to_llvm(context)
+        return llvm.core.Type.int(1)
 
 class NumericType(NamedType):
     """
@@ -670,15 +720,26 @@ class FloatType(NumericType):
     def comparison_type_list(self):
         return self.subtype_list + [self.itemsize]
 
+    def __eq__(self, other):
+        return isinstance(other, FloatType) and self.itemsize == other.itemsize
+
+    __hash__ = NumericType.__hash__
+
     def to_llvm(self, context):
         if self.itemsize == 4:
             return lc.Type.float()
         elif self.itemsize == 8:
             return lc.Type.double()
         else:
-            # Note: what about fp80/fp96?
-            assert self.itemsize == 16
-            return lc.Type.fp128()
+            is_ppc, is_x86 = get_target_triple()
+            if self.itemsize == 16:
+                if is_ppc:
+                    return lc.Type.ppc_fp128()
+                else:
+                    return lc.Type.fp128()
+            else:
+                assert self.itemsize == 10 and is_x86
+                return lc.Type.x86_fp80()
 
 class ComplexType(NumericType):
     is_complex = True
@@ -760,11 +821,12 @@ class FunctionType(Type):
 
     struct_by_reference = False
 
-    def __init__(self, return_type, args, name=None, **kwds):
+    def __init__(self, return_type, args, name=None, is_vararg=False, **kwds):
         super(FunctionType, self).__init__(**kwds)
         self.return_type = return_type
         self.args = args
         self.name = name
+        self.is_vararg = is_vararg
 
     def to_llvm(self, context):
         assert self.return_type is not None
@@ -779,7 +841,7 @@ class FunctionType(Type):
         if self.is_vararg:
             args.append("...")
         if self.name:
-            namestr = self.name + ' '
+            namestr = self.name
         else:
             namestr = ''
         return "%s (*%s)(%s)" % (self.return_type, namestr, ", ".join(args))
@@ -862,9 +924,9 @@ def _sort_types_key(field_type):
     elif field_type.is_numeric or field_type.is_struct:
         return field_type.rank
     elif field_type.is_vector:
-        return _sort_key(field_type.element_type) * field_type.vector_size
+        return _sort_types_key(field_type.element_type) * field_type.vector_size
     elif field_type.is_carray:
-        return _sort_key(field_type.base_type) * field_type.size
+        return _sort_types_key(field_type.base_type) * field_type.size
     elif field_type.is_pointer or field_type.is_object or field_type.is_array:
         return 8
     else:
@@ -903,8 +965,12 @@ class struct(Type):
     >>> struct(a=int32, b=int32, name='Foo') # unordered struct
     struct Foo { int32 a, int32 b }
 
-    >>> struct(a=complex128, b=complex64, c=struct(f1=double, f2=double, f3=int32))
+    >>> S = struct(a=complex128, b=complex64, c=struct(f1=double, f2=double, f3=int32))
+    >>> S
     struct { struct { double f1, double f2, int32 f3 } c, complex128 a, complex64 b }
+
+    >>> S.offsetof('a')
+    24
     """
 
     is_struct = True
@@ -919,12 +985,16 @@ class struct(Type):
             fields = sort_types(kwargs)
 
         self.fields = list(fields)
-        self.rank = sum(_sort_key(field) for field in self.fields)
         self.name = name
         self.readonly = readonly
         self.fielddict = dict(self.fields)
         self.packed = packed
 
+        self.update_mutated()
+
+    def copy(self):
+        return struct(self.fields, self.name, self.readonly, self.packed)
+
     def __repr__(self):
         if self.name:
             name = self.name + ' '
@@ -947,6 +1017,10 @@ class struct(Type):
     def comparison_type_list(self):
         return self.fields
 
+    @property
+    def subtype_list(self):
+        return [field[1] for field in self.fields]
+
     def is_prefix(self, other_struct):
         other_fields = other_struct.fields[:len(self.fields)]
         return self.fields == other_fields
@@ -955,6 +1029,19 @@ class struct(Type):
         assert name not in self.fielddict
         self.fielddict[name] = type
         self.fields.append((name, type))
+        self.mutated = True
+
+    def update_mutated(self):
+        self.rank = sum([_sort_key(field) for field in self.fields])
+        self.mutated = False
+
+    def offsetof(self, field_name):
+        """
+        Compute the offset of a field. Must be used only after mutation has
+        finished.
+        """
+        ctype = self.to_ctypes()
+        return getattr(ctype, field_name).offset
 
 def getsize(ctypes_name, default):
     try:
@@ -962,6 +1049,12 @@ def getsize(ctypes_name, default):
     except ImportError:
         return default
 
+def get_target_triple():
+    target_machine = llvm.ee.TargetMachine.new()
+    is_ppc = target_machine.triple.startswith("ppc")
+    is_x86 = target_machine.triple.startswith("x86")
+    return is_ppc, is_x86
+
 #
 ### Internal types
 #
@@ -998,6 +1091,11 @@ ulong = IntType(name="unsigned long", rank=5.5, typecode='L', signed=False)
 ulonglong = IntType(name="unsigned PY_LONG_LONG", rank=8.5,
                     typecode='Q', signed=False)
 
+float_ = FloatType(name="float", rank=20, itemsize=4)
+double = FloatType(name="double", rank=21, itemsize=8)
+longdouble = FloatType(name="long double", rank=22,
+                       itemsize=ctypes.sizeof(ctypes.c_longdouble))
+
 bool_ = BoolType()
 object_ = ObjectType()
 
@@ -1011,17 +1109,16 @@ uint16 = IntType(name="uint16", rank=2.5, signed=False, itemsize=2)
 uint32 = IntType(name="uint32", rank=4.5, signed=False, itemsize=4)
 uint64 = IntType(name="uint64", rank=8.5, signed=False, itemsize=8)
 
-float32 = float_ = FloatType(name="float", rank=10, itemsize=4)
-float64 = double = FloatType(name="double", rank=12, itemsize=8)
-float128 = longdouble = FloatType(name="long double", rank=14,
-                                  itemsize=16)
+float32 = FloatType(name="float32", rank=20, itemsize=4)
+float64 = FloatType(name="float64", rank=21, itemsize=8)
+float128 = FloatType(name="float128", rank=22, itemsize=16)
 
 complex64 = ComplexType(name="complex64", base_type=float32,
-                        rank=16, itemsize=8)
+                        rank=30, itemsize=8)
 complex128 = ComplexType(name="complex128", base_type=float64,
-                         rank=18, itemsize=16)
+                         rank=31, itemsize=16)
 complex256 = ComplexType(name="complex256", base_type=float128,
-                         rank=20, itemsize=32)
+                         rank=32, itemsize=32)
 
 integral = []
 native_integral = []
@@ -1030,6 +1127,9 @@ complextypes = []
 
 for typename in __all__:
     minitype = globals()[typename]
+    if minitype is None:
+        continue
+
     if minitype.is_int:
         integral.append(minitype)
     elif minitype.is_float:
