@@ -107,7 +107,7 @@ NumbaError: 10:5: Dynamic closures not yet supported, use @jit
 >>> error3()
 Traceback (most recent call last):
     ...
-NumbaError: 16:4: Local variable  'inner' is not bound yet
+NumbaError: 16:4: local variable 'inner' referenced before assignment
 >>> error4()
 Traceback (most recent call last):
     ...
@@ -120,6 +120,7 @@ NumbaError: Got multiple values for positional argument 'a'
 Test closures
 
 >>> closure1().__name__
+Warning 40:4: Unused variable 'a'
 'inner'
 >>> closure1()()
 Traceback (most recent call last):
@@ -179,8 +180,6 @@ def closure_arg(a):
 
 __doc__ += \
 """
-This test doesn't work yet...
-
 >>> closure1 = closure_arg(1)
 >>> closure1.__name__
 'closure1'
@@ -209,6 +208,7 @@ This test doesn't work yet...
 1 3 5 7 13 18.0
 """
 
+
 @autojit
 def closure_arg_simple(a):
     @jit('object_(object_)')
@@ -278,7 +278,7 @@ __doc__ += """
 >>> wrong_restype()
 Traceback (most recent call last):
     ...
-NumbaError: 1:0: Function with non-void return does not return a value
+NumbaError: 272:4: Function with non-void return does not return a value
 """
 
 #
@@ -310,6 +310,72 @@ Traceback (most recent call last):
 NumbaError: 301:5: Expected 1 arguments type(s), got 0
 """
 
+@autojit
+def get_closure(arg):
+    @void()
+    def closure():
+        print arg
+
+    return closure
+
+@autojit
+def test_call_closure():
+    closure = get_closure(10.0)
+    closure() # TODO: This still goes through the object layer, amend
+
+__doc__ += """
+>>> test_call_closure()
+10.0
+"""
+
+@autojit
+def test_closure_loop():
+    """
+    >>> test_closure_loop()
+    0 3
+    1 3
+    2 3
+    <BLANKLINE>
+    0 3
+    1 3
+    2 3
+    """
+    cellvar = 3
+
+    @jit(void())
+    def inner():
+        for i in range(cellvar):
+            print i, cellvar
+
+        print
+
+        for i in range(cellvar):
+            for j in range(cellvar):
+                if i == j:
+                    print i, cellvar
+
+    inner()
+
+@autojit(locals=dict(var=int_), warn=False)
+def test_closure_outer_locals():
+    """
+    >>> test_closure_outer_locals()
+    """
+    var = 10
+    @jit(void())
+    def inner():
+        var = "hello"
+
+    inner()
+
 if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
\ No newline at end of file
+#    closure1 = closure_arg(1)
+#    print closure1.__name__
+#    test_call_closure()
+#    closure4()
+#    signature_dec()()
+#    test_closure_outer_locals()
+#    test_closure_loop()
+    test_closure_outer_locals()
+#    import numba
+#    numba.testmod()
\ No newline at end of file
