@@ -9,6 +9,10 @@ from numba import *
 def print_(value):
     print value
 
+@autojit(backend='ast', nopython=True)
+def print_nopython(value):
+    print "value", value
+
 @autojit(backend='ast')
 def print_to_stream(stream, value):
     print >>stream, value
@@ -41,7 +45,8 @@ class TestPrint(unittest.TestCase):
         temp_out = StringIO.StringIO()
         print_no_newline(temp_out, 14.1)
         data = temp_out.getvalue()
-        assert data == "14.1", repr(data)
+        assert data == "14.1 ", repr(data)
 
 if __name__ == "__main__":
+    print_nopython(10)
     unittest.main()
\ No newline at end of file
