@@ -5,7 +5,11 @@ import functools
 
 from nose.tools import nottest
 import nose.plugins.skip
-from numba.decorators import jit as jit_, autojit
+import numba
+from numba import *
+import doctest_support
+
+jit_ = jit
 
 import __builtin__
 
@@ -27,6 +31,9 @@ def main():
     if '-d' in sys.argv:
         logging.getLogger().setLevel(logging.DEBUG)
         sys.argv.remove('-d')
+    if '-D' in sys.argv:
+        logging.getLogger().setLevel(logging.NOTSET)
+        sys.argv.remove('-D')
     unittest.main()
 
 class StdoutReplacer(object):
@@ -38,4 +45,19 @@ class StdoutReplacer(object):
     def __exit__(self, *args):
         sys.stdout = self.out
 
-from .bytecode.test_support import ByteCodeTestCase
\ No newline at end of file
+from bytecode.test_support import ByteCodeTestCase
+
+def testmod(module=None, runit=False):
+    """
+    Tests a doctest modules with numba functions. When run in nosetests, only
+    populates module.__test__, when run as main, runs the doctests.
+    """
+    if module is None:
+        modname = sys._getframe(1).f_globals['__name__']
+        module = __import__(modname)
+    else:
+        modname = module.__name__
+
+    doctest_support.testmod(module, run_doctests=runit or modname == '__main__')
+    #if modname == '__main__':
+    #    numba.nose_run(mod)
\ No newline at end of file
